@@ -1,4 +1,4 @@
-/*	$OpenBSD: bgpd.c,v 1.245 2022/06/09 16:45:19 claudio Exp $ */
+/*	$OpenBSD: bgpd.c,v 1.248 2022/06/23 13:09:03 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -935,11 +935,11 @@ dispatch_imsg(struct imsgbuf *ibuf, int idx, struct bgpd_config *conf)
 			else
 				kr_show_route(&imsg);
 			break;
-		case IMSG_IFINFO:
+		case IMSG_SESSION_DEPENDON:
 			if (idx != PFD_PIPE_SESSION)
-				log_warnx("IFINFO request not from SE");
+				log_warnx("DEPENDON request not from SE");
 			else if (imsg.hdr.len != IMSG_HEADER_SIZE + IFNAMSIZ)
-				log_warnx("IFINFO request with wrong len");
+				log_warnx("DEPENDON request with wrong len");
 			else
 				kr_ifinfo(imsg.data);
 			break;
@@ -1114,25 +1114,19 @@ send_network(int type, struct network_config *net, struct filter_set_head *h)
 }
 
 int
-bgpd_filternexthop(struct kroute *kr, struct kroute6 *kr6)
+bgpd_filternexthop(struct kroute_full *kf)
 {
 	/* kernel routes are never filtered */
-	if (kr && kr->flags & F_KERNEL && kr->prefixlen != 0)
-		return (0);
-	if (kr6 && kr6->flags & F_KERNEL && kr6->prefixlen != 0)
+	if (kf->flags & F_KERNEL && kf->prefixlen != 0)
 		return (0);
 
 	if (cflags & BGPD_FLAG_NEXTHOP_BGP) {
-		if (kr && kr->flags & F_BGPD_INSERTED)
-			return (0);
-		if (kr6 && kr6->flags & F_BGPD_INSERTED)
+		if (kf->flags & F_BGPD)
 			return (0);
 	}
 
 	if (cflags & BGPD_FLAG_NEXTHOP_DEFAULT) {
-		if (kr && kr->prefixlen == 0)
-			return (0);
-		if (kr6 && kr6->prefixlen == 0)
+		if (kf->prefixlen == 0)
 			return (0);
 	}
 
