@@ -1,4 +1,4 @@
-/*	$OpenBSD: bgpd.h,v 1.428 2022/06/09 16:45:19 claudio Exp $ */
+/*	$OpenBSD: bgpd.h,v 1.441 2022/07/11 17:08:21 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -26,7 +26,6 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <net/if.h>
-#include <net/pfkeyv2.h>
 
 #include <poll.h>
 #include <stdarg.h>
@@ -84,13 +83,13 @@
 #endif
 #define	SOCKET_NAME			RUNSTATEDIR "/bgpd.sock"
 
-#define	F_BGPD_INSERTED		0x0001
+#define	F_BGPD			0x0001
 #define	F_KERNEL		0x0002
 #define	F_CONNECTED		0x0004
 #define	F_NEXTHOP		0x0008
 #define	F_DOWN			0x0010
 #define	F_STATIC		0x0020
-#define	F_DYNAMIC		0x0040
+#define	F_BGPD_INSERTED		0x0040
 #define	F_REJECT		0x0080
 #define	F_BLACKHOLE		0x0100
 #define	F_LONGER		0x0200
@@ -318,6 +317,20 @@ struct bgpd_config {
 
 extern int cmd_opts;
 
+enum addpath_mode {
+	ADDPATH_EVAL_NONE,
+	ADDPATH_EVAL_BEST,
+	ADDPATH_EVAL_ECMP,
+	ADDPATH_EVAL_AS_WIDE,
+	ADDPATH_EVAL_ALL,
+};
+
+struct addpath_eval {
+	enum addpath_mode	mode;
+	int			extrapaths;
+	int			maxpaths;
+};
+
 enum export_type {
 	EXPORT_UNSET,
 	EXPORT_NONE,
@@ -339,6 +352,18 @@ enum auth_method {
 	AUTH_IPSEC_IKE_AH
 };
 
+enum auth_alg {
+	AUTH_AALG_NONE,
+	AUTH_AALG_SHA1HMAC,
+	AUTH_AALG_MD5HMAC,
+};
+
+enum auth_enc_alg {
+	AUTH_EALG_NONE,
+	AUTH_EALG_3DESCBC,
+	AUTH_EALG_AES,
+};
+
 struct peer_auth {
 	char			md5key[TCP_MD5_KEY_LEN];
 	char			auth_key_in[IPSEC_AUTH_KEY_LEN];
@@ -348,13 +373,13 @@ struct peer_auth {
 	uint32_t		spi_in;
 	uint32_t		spi_out;
 	enum auth_method	method;
+	enum auth_alg		auth_alg_in;
+	enum auth_alg		auth_alg_out;
+	enum auth_enc_alg	enc_alg_in;
+	enum auth_enc_alg	enc_alg_out;
 	uint8_t			md5key_len;
-	uint8_t			auth_alg_in;
-	uint8_t			auth_alg_out;
 	uint8_t			auth_keylen_in;
 	uint8_t			auth_keylen_out;
-	uint8_t			enc_alg_in;
-	uint8_t			enc_alg_out;
 	uint8_t			enc_keylen_in;
 	uint8_t			enc_keylen_out;
 };
@@ -370,9 +395,11 @@ struct capabilities {
 	int8_t	as4byte;		/* 4-byte ASnum, RFC 4893 */
 	int8_t	enhanced_rr;		/* enhanced route refresh, RFC 7313 */
 	int8_t	add_path[AID_MAX];	/* ADD_PATH, RFC 7911 */
+	uint8_t	role;			/* Open Policy, RFC 9234 */
+	int8_t	role_ena;		/* 1 for enable, 2 for enforce */
 };
 
-/* flags for RFC4724 - graceful restart */
+/* flags for RFC 4724 - graceful restart */
 #define	CAPA_GR_PRESENT		0x01
 #define	CAPA_GR_RESTART		0x02
 #define	CAPA_GR_FORWARD		0x04
@@ -381,17 +408,25 @@ struct capabilities {
 #define	CAPA_GR_R_FLAG		0x8000
 #define	CAPA_GR_F_FLAG		0x80
 
-/* flags for RFC7911 - enhanced router refresh */
+/* flags for RFC 7911 - enhanced router refresh */
 #define	CAPA_AP_RECV		0x01
 #define	CAPA_AP_SEND		0x02
 #define	CAPA_AP_BIDIR		0x03
 
+/* values for RFC 9234 - BGP Open Policy */
+#define CAPA_ROLE_PROVIDER	0x00
+#define CAPA_ROLE_RS		0x01
+#define CAPA_ROLE_RS_CLIENT	0x02
+#define CAPA_ROLE_CUSTOMER	0x03
+#define CAPA_ROLE_PEER		0x04
+
 struct peer_config {
 	struct bgpd_addr	 remote_addr;
 	struct bgpd_addr	 local_addr_v4;
 	struct bgpd_addr	 local_addr_v6;
 	struct peer_auth	 auth;
 	struct capabilities	 capabilities;
+	struct addpath_eval	 eval;
 	char			 group[PEER_DESCR_LEN];
 	char			 descr[PEER_DESCR_LEN];
 	char			 reason[REASON_LEN];
@@ -571,6 +606,7 @@ enum imsg_type {
 	IMSG_SESSION_STALE,
 	IMSG_SESSION_FLUSH,
 	IMSG_SESSION_RESTARTED,
+	IMSG_SESSION_DEPENDON,
 	IMSG_PFKEY_RELOAD,
 	IMSG_MRT_OPEN,
 	IMSG_MRT_REOPEN,
@@ -585,7 +621,6 @@ enum imsg_type {
 	IMSG_PFTABLE_REMOVE,
 	IMSG_PFTABLE_COMMIT,
 	IMSG_REFRESH,
-	IMSG_IFINFO,
 	IMSG_DEMOTE,
 	IMSG_XON,
 	IMSG_XOFF
@@ -655,13 +690,13 @@ enum suberr_rrefresh {
 	ERR_RR_INV_LEN = 1
 };
 
-struct kroute_node;
-struct kroute6_node;
-struct knexthop_node;
+struct kroute;
+struct kroute6;
+struct knexthop;
 struct kredist_node;
-RB_HEAD(kroute_tree, kroute_node);
-RB_HEAD(kroute6_tree, kroute6_node);
-RB_HEAD(knexthop_tree, knexthop_node);
+RB_HEAD(kroute_tree, kroute);
+RB_HEAD(kroute6_tree, kroute6);
+RB_HEAD(knexthop_tree, knexthop);
 RB_HEAD(kredist_tree, kredist_node);
 
 struct ktable {
@@ -683,34 +718,13 @@ struct kroute_full {
 	struct bgpd_addr	prefix;
 	struct bgpd_addr	nexthop;
 	char			label[RTLABEL_LEN];
+	uint32_t		mplslabel;
 	uint16_t		flags;
 	u_short			ifindex;
 	uint8_t			prefixlen;
 	uint8_t			priority;
 };
 
-struct kroute {
-	struct in_addr	prefix;
-	struct in_addr	nexthop;
-	uint32_t	mplslabel;
-	uint16_t	flags;
-	uint16_t	labelid;
-	u_short		ifindex;
-	uint8_t		prefixlen;
-	uint8_t		priority;
-};
-
-struct kroute6 {
-	struct in6_addr	prefix;
-	struct in6_addr	nexthop;
-	uint32_t	mplslabel;
-	uint16_t	flags;
-	uint16_t	labelid;
-	u_short		ifindex;
-	uint8_t		prefixlen;
-	uint8_t		priority;
-};
-
 struct kroute_nexthop {
 	struct bgpd_addr	nexthop;
 	struct bgpd_addr	gateway;
@@ -720,15 +734,8 @@ struct kroute_nexthop {
 	uint8_t			netlen;
 };
 
-struct kif {
+struct session_dependon {
 	char			 ifname[IFNAMSIZ];
-	uint64_t		 baudrate;
-	u_int			 rdomain;
-	int			 flags;
-	u_short			 ifindex;
-	uint8_t			 if_type;
-	uint8_t			 link_state;
-	uint8_t			 nh_reachable;	/* for nexthop verification */
 	uint8_t			 depend_state;	/* for session depend on */
 };
 
@@ -768,10 +775,7 @@ struct ctl_show_interface {
 struct ctl_show_nexthop {
 	struct bgpd_addr		addr;
 	struct ctl_show_interface	iface;
-	union {
-		struct kroute		kr4;
-		struct kroute6		kr6;
-	} kr;
+	struct kroute_full		kr;
 	uint8_t				valid;
 	uint8_t				krvalid;
 };
@@ -798,13 +802,16 @@ struct ctl_neighbor {
 	int			is_group;
 };
 
-#define	F_PREF_ELIGIBLE	0x01
-#define	F_PREF_BEST	0x02
-#define	F_PREF_INTERNAL	0x04
-#define	F_PREF_ANNOUNCE	0x08
-#define	F_PREF_STALE	0x10
-#define	F_PREF_INVALID	0x20
-#define	F_PREF_PATH_ID	0x40
+#define	F_PREF_ELIGIBLE	0x001
+#define	F_PREF_BEST	0x002
+#define	F_PREF_INTERNAL	0x004
+#define	F_PREF_ANNOUNCE	0x008
+#define	F_PREF_STALE	0x010
+#define	F_PREF_INVALID	0x020
+#define	F_PREF_PATH_ID	0x040
+#define	F_PREF_OTC_LOOP	0x080
+#define	F_PREF_ECMP	0x100
+#define	F_PREF_AS_WIDE	0x200
 
 struct ctl_show_rib {
 	struct bgpd_addr	true_nexthop;
@@ -822,6 +829,7 @@ struct ctl_show_rib {
 	uint8_t			prefixlen;
 	uint8_t			origin;
 	uint8_t			validation_state;
+	int8_t			dmetric;
 	/* plus an aspath */
 };
 
@@ -979,7 +987,7 @@ struct filter_peers {
 #define EXT_COMMUNITY_TRANS_IPV4	0x01	/* IPv4 specific */
 #define EXT_COMMUNITY_TRANS_FOUR_AS	0x02	/* 4 octet AS specific */
 #define EXT_COMMUNITY_TRANS_OPAQUE	0x03	/* opaque ext community */
-#define EXT_COMMUNITY_TRANS_EVPN	0x06	/* EVPN RFC7432 */
+#define EXT_COMMUNITY_TRANS_EVPN	0x06	/* EVPN RFC 7432 */
 /* extended types non-transitive */
 #define EXT_COMMUNITY_NON_TRANS_TWO_AS	0x40	/* 2 octet AS specific */
 #define EXT_COMMUNITY_NON_TRANS_IPV4	0x41	/* IPv4 specific */
@@ -987,7 +995,7 @@ struct filter_peers {
 #define EXT_COMMUNITY_NON_TRANS_OPAQUE	0x43	/* opaque ext community */
 #define EXT_COMMUNITY_UNKNOWN		-1
 
-/* BGP Origin Validation State Extended Community RFC8097 */
+/* BGP Origin Validation State Extended Community RFC 8097 */
 #define EXT_COMMUNITY_SUBTYPE_OVS	0
 #define EXT_COMMUNITY_OVS_VALID		0
 #define EXT_COMMUNITY_OVS_NOTFOUND	1
@@ -1267,7 +1275,7 @@ void		 send_nexthop_update(struct kroute_nexthop *);
 void		 send_imsg_session(int, pid_t, void *, uint16_t);
 int		 send_network(int, struct network_config *,
 		     struct filter_set_head *);
-int		 bgpd_filternexthop(struct kroute *, struct kroute6 *);
+int		 bgpd_filternexthop(struct kroute_full *);
 void		 set_pollfd(struct pollfd *, struct imsgbuf *);
 int		 handle_pollfd(struct pollfd *, struct imsgbuf *);
 
@@ -1314,7 +1322,6 @@ void		 kr_show_route(struct imsg *);
 void		 kr_ifinfo(char *);
 void		 kr_net_reload(u_int, uint64_t, struct network_head *);
 int		 kr_reload(void);
-struct in6_addr	*prefixlen2mask6(uint8_t prefixlen);
 int		 get_mpe_config(const char *, u_int *, u_int *);
 
 /* log.c */
@@ -1412,6 +1419,7 @@ const char	*log_rd(uint64_t);
 const char	*log_ext_subtype(int, uint8_t);
 const char	*log_reason(const char *);
 const char	*log_rtr_error(enum rtr_error);
+const char	*log_policy(uint8_t);
 int		 aspath_snprint(char *, size_t, void *, uint16_t);
 int		 aspath_asprint(char **, void *, uint16_t);
 size_t		 aspath_strlen(void *, uint16_t);
@@ -1432,10 +1440,10 @@ int		 nlri_get_vpn6(u_char *, uint16_t, struct bgpd_addr *,
 		     uint8_t *, int);
 int		 prefix_compare(const struct bgpd_addr *,
 		    const struct bgpd_addr *, int);
-in_addr_t	 prefixlen2mask(uint8_t);
 void		 inet4applymask(struct in_addr *, const struct in_addr *, int);
 void		 inet6applymask(struct in6_addr *, const struct in6_addr *,
 		    int);
+void		 applymask(struct bgpd_addr *, const struct bgpd_addr *, int);
 const char	*aid2str(uint8_t);
 int		 aid2afi(uint8_t, uint16_t *, uint8_t *);
 int		 afi2aid(uint16_t, uint8_t, uint8_t *);
@@ -1517,8 +1525,10 @@ static const char * const suberr_open_names[] = {
 	"authentication error",
 	"unacceptable holdtime",
 	"unsupported capability",
-	"group membership conflict",	/* draft-ietf-idr-bgp-multisession-07 */
-	"group membership required"	/* draft-ietf-idr-bgp-multisession-07 */
+	NULL,
+	NULL,
+	NULL,
+	"role mismatch",
 };
 
 static const char * const suberr_fsm_names[] = {
