@@ -1,4 +1,4 @@
-/*	$OpenBSD: config.c,v 1.101 2022/02/23 11:20:35 claudio Exp $ */
+/*	$OpenBSD: config.c,v 1.102 2022/06/05 12:43:13 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004, 2005 Henning Brauer <henning@openbsd.org>
@@ -229,9 +229,9 @@ merge_config(struct bgpd_config *xconf, struct bgpd_config *conf)
 	/* adjust FIB priority if changed */
 	/* if xconf is uninitialized we get RTP_NONE */
 	if (xconf->fib_priority != conf->fib_priority) {
-		kr_fib_decouple_all(xconf->fib_priority);
-		kr_fib_update_prio_all(conf->fib_priority);
-		kr_fib_couple_all(conf->fib_priority);
+		kr_fib_decouple_all();
+		kr_fib_prio_set(conf->fib_priority);
+		kr_fib_couple_all();
 	}
 
 	/* take over the easy config changes */
