@@ -1,4 +1,4 @@
-/*	$OpenBSD: pfkey.c,v 1.62 2022/02/06 09:51:19 claudio Exp $ */
+/*	$OpenBSD: pfkey.c,v 1.65 2022/06/16 09:51:07 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -469,6 +469,9 @@ pfkey_reply(int sd, uint32_t *spi)
 			return (0);
 		else {
 			log_warn("pfkey");
+			/* discard error message */
+			if (read(sd, &hdr, sizeof(hdr)) == -1)
+				log_warn("pfkey read");
 			return (-1);
 		}
 	}
@@ -564,7 +567,7 @@ pfkey_md5sig_establish(struct peer *p)
 	return (0);
 
 fail:
-	log_peer_warn(&p->conf, "%s: failed to insert md5sig", __func__);
+	log_peer_warn(&p->conf, "failed to insert md5sig");
 	return (-1);
 }
 
@@ -586,10 +589,36 @@ pfkey_md5sig_remove(struct peer *p)
 	return (0);
 
 fail:
-	log_peer_warn(&p->conf, "%s: failed to remove md5sig", __func__);
+	log_peer_warn(&p->conf, "failed to remove md5sig");
 	return (-1);
 }
 
+static uint8_t
+pfkey_auth_alg(enum auth_alg alg)
+{
+	switch (alg) {
+	case AUTH_AALG_SHA1HMAC:
+		return SADB_AALG_SHA1HMAC;
+	case AUTH_AALG_MD5HMAC:
+		return SADB_AALG_MD5HMAC;
+	default:
+		return SADB_AALG_NONE;
+	}
+}
+
+static uint8_t
+pfkey_enc_alg(enum auth_enc_alg alg)
+{
+	switch (alg) {
+	case AUTH_EALG_3DESCBC:
+		return SADB_EALG_3DESCBC;
+	case AUTH_EALG_AES:
+		return SADB_X_EALG_AES;
+	default:
+		return SADB_AALG_NONE;
+	}
+}
+
 static int
 pfkey_ipsec_establish(struct peer *p)
 {
@@ -616,10 +645,10 @@ pfkey_ipsec_establish(struct peer *p)
 		if (pfkey_send(pfkey_fd, satype, SADB_ADD, 0,
 		    local_addr, &p->conf.remote_addr,
 		    p->conf.auth.spi_out,
-		    p->conf.auth.auth_alg_out,
+		    pfkey_auth_alg(p->conf.auth.auth_alg_out),
 		    p->conf.auth.auth_keylen_out,
 		    p->conf.auth.auth_key_out,
-		    p->conf.auth.enc_alg_out,
+		    pfkey_enc_alg(p->conf.auth.enc_alg_out),
 		    p->conf.auth.enc_keylen_out,
 		    p->conf.auth.enc_key_out,
 		    0, 0) == -1)
@@ -629,10 +658,10 @@ pfkey_ipsec_establish(struct peer *p)
 		if (pfkey_send(pfkey_fd, satype, SADB_ADD, 0,
 		    &p->conf.remote_addr, local_addr,
 		    p->conf.auth.spi_in,
-		    p->conf.auth.auth_alg_in,
+		    pfkey_auth_alg(p->conf.auth.auth_alg_in),
 		    p->conf.auth.auth_keylen_in,
 		    p->conf.auth.auth_key_in,
-		    p->conf.auth.enc_alg_in,
+		    pfkey_enc_alg(p->conf.auth.enc_alg_in),
 		    p->conf.auth.enc_keylen_in,
 		    p->conf.auth.enc_key_in,
 		    0, 0) == -1)
@@ -675,10 +704,10 @@ pfkey_ipsec_establish(struct peer *p)
 	return (0);
 
 fail_key:
-	log_peer_warn(&p->conf, "%s: failed to insert ipsec key", __func__);
+	log_peer_warn(&p->conf, "failed to insert ipsec key");
 	return (-1);
 fail_flow:
-	log_peer_warn(&p->conf, "%s: failed to insert flow", __func__);
+	log_peer_warn(&p->conf, "failed to insert ipsec flow");
 	return (-1);
 }
 
@@ -748,10 +777,10 @@ pfkey_ipsec_remove(struct peer *p)
 	return (0);
 
 fail_key:
-	log_peer_warn(&p->conf, "%s: failed to remove ipsec key", __func__);
+	log_peer_warn(&p->conf, "failed to remove ipsec key");
 	return (-1);
 fail_flow:
-	log_peer_warn(&p->conf, "%s: failed to remove flow", __func__);
+	log_peer_warn(&p->conf, "failed to remove ipsec flow");
 	return (-1);
 }
 
