@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde_decide.c,v 1.91 2022/03/22 10:53:08 claudio Exp $ */
+/*	$OpenBSD: rde_decide.c,v 1.95 2022/07/11 16:46:41 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Claudio Jeker <claudio@openbsd.org>
@@ -27,6 +27,7 @@
 #include "log.h"
 
 int	prefix_cmp(struct prefix *, struct prefix *, int *);
+void	prefix_set_dmetric(struct prefix *, struct prefix *);
 void	prefix_insert(struct prefix *, struct prefix *, struct rib_entry *);
 void	prefix_remove(struct prefix *, struct rib_entry *);
 /*
@@ -106,7 +107,14 @@ void	prefix_remove(struct prefix *, struct rib_entry *);
  * Compare two prefixes with equal pt_entry. Returns an integer greater than or
  * less than 0, according to whether the prefix p1 is more or less preferred
  * than the prefix p2. p1 should be used for the new prefix and p2 for a
- * already added prefix.
+ * already added prefix. The absolute value returned specifies the similarity
+ * of the prefixes.
+ *   1: prefixes differ because of validity
+ *   2: prefixes don't belong in any multipath set
+ *   3: prefixes belong only in the as-wide multipath set
+ *   4: prefixes belong in both the ecmp and as-wide multipath set
+ *   TODO: maybe we also need a strict ecmp set that requires
+ *   prefixes to e.g. equal ASPATH or equal neighbor-as (like for MED).
  */
 int
 prefix_cmp(struct prefix *p1, struct prefix *p2, int *testall)
@@ -116,6 +124,7 @@ prefix_cmp(struct prefix *p1, struct prefix *p2, int *testall)
 	struct attr		*a;
 	uint32_t		 p1id, p2id;
 	int			 p1cnt, p2cnt, i;
+	int			 rv = 1;
 
 	/*
 	 * If a match happens before the MED check then the list is
@@ -129,9 +138,9 @@ prefix_cmp(struct prefix *p1, struct prefix *p2, int *testall)
 	*testall = 0;
 
 	if (p1 == NULL)
-		return -1;
+		return -rv;
 	if (p2 == NULL)
-		return 1;
+		return rv;
 
 	asp1 = prefix_aspath(p1);
 	asp2 = prefix_aspath(p2);
@@ -140,15 +149,15 @@ prefix_cmp(struct prefix *p1, struct prefix *p2, int *testall)
 
 	/* pathes with errors are not eligible */
 	if (asp1 == NULL || asp1->flags & F_ATTR_PARSE_ERR)
-		return -1;
+		return -rv;
 	if (asp2 == NULL || asp2->flags & F_ATTR_PARSE_ERR)
-		return 1;
+		return rv;
 
 	/* only loop free pathes are eligible */
 	if (asp1->flags & F_ATTR_LOOP)
-		return -1;
+		return -rv;
 	if (asp2->flags & F_ATTR_LOOP)
-		return 1;
+		return rv;
 
 	/*
 	 * 1. check if prefix is eligible a.k.a reachable
@@ -157,24 +166,31 @@ prefix_cmp(struct prefix *p1, struct prefix *p2, int *testall)
 	 */
 	if (prefix_nexthop(p2) != NULL &&
 	    prefix_nexthop(p2)->state != NEXTHOP_REACH)
-		return 1;
+		return rv;
 	if (prefix_nexthop(p1) != NULL &&
 	    prefix_nexthop(p1)->state != NEXTHOP_REACH)
-		return -1;
+		return -rv;
+
+	/* bump rv, from here on prefix is considered valid */
+	rv++;
 
 	/* 2. local preference of prefix, bigger is better */
 	if (asp1->lpref > asp2->lpref)
-		return 1;
+		return rv;
 	if (asp1->lpref < asp2->lpref)
-		return -1;
+		return -rv;
 
 	/* 3. aspath count, the shorter the better */
-	if ((asp2->aspath->ascnt - asp1->aspath->ascnt) != 0)
-		return (asp2->aspath->ascnt - asp1->aspath->ascnt);
+	if (asp1->aspath->ascnt < asp2->aspath->ascnt)
+		return rv;
+	if (asp1->aspath->ascnt > asp2->aspath->ascnt)
+		return -rv;
 
 	/* 4. origin, the lower the better */
-	if ((asp2->origin - asp1->origin) != 0)
-		return (asp2->origin - asp1->origin);
+	if (asp1->origin < asp2->origin)
+		return rv;
+	if (asp1->origin > asp2->origin)
+		return -rv;
 
 	/*
 	 * 5. MED decision
@@ -189,9 +205,9 @@ prefix_cmp(struct prefix *p1, struct prefix *p2, int *testall)
 			*testall = 2;
 		/* lowest value wins */
 		if (asp1->med < asp2->med)
-			return 1;
+			return rv;
 		if (asp1->med > asp2->med)
-			return -1;
+			return -rv;
 	}
 
 	if (!(rde_decisionflags() & BGPD_FLAG_DECISION_MED_ALWAYS))
@@ -204,11 +220,14 @@ prefix_cmp(struct prefix *p1, struct prefix *p2, int *testall)
 	 */
 	if (peer1->conf.ebgp != peer2->conf.ebgp) {
 		if (peer1->conf.ebgp) /* peer1 is EBGP other is lower */
-			return 1;
+			return rv;
 		else if (peer2->conf.ebgp) /* peer2 is EBGP */
-			return -1;
+			return -rv;
 	}
 
+	/* bump rv, as-wide multipath */
+	rv++;
+
 	/*
 	 * 7. local tie-breaker, this weight is here to tip equal long AS
 	 * paths in one or the other direction. It happens more and more
@@ -217,21 +236,24 @@ prefix_cmp(struct prefix *p1, struct prefix *p2, int *testall)
 	 * decision process.
 	 */
 	if (asp1->weight > asp2->weight)
-		return 1;
+		return rv;
 	if (asp1->weight < asp2->weight)
-		return -1;
+		return -rv;
 
 	/* 8. nexthop costs. NOT YET -> IGNORE */
 
+	/* bump rv, equal cost multipath */
+	rv++;
+
 	/*
 	 * 9. older route (more stable) wins but only if route-age
 	 * evaluation is enabled.
 	 */
 	if (rde_decisionflags() & BGPD_FLAG_DECISION_ROUTEAGE) {
 		if (p1->lastchange < p2->lastchange) /* p1 is older */
-			return 1;
+			return rv;
 		if (p1->lastchange > p2->lastchange)
-			return -1;
+			return -rv;
 	}
 
 	/* 10. lowest BGP Id wins, use ORIGINATOR_ID if present */
@@ -246,9 +268,9 @@ prefix_cmp(struct prefix *p1, struct prefix *p2, int *testall)
 	} else
 		p2id = peer2->remote_bgpid;
 	if (p1id < p2id)
-		return 1;
+		return rv;
 	if (p1id > p2id)
-		return -1;
+		return -rv;
 
 	/* 11. compare CLUSTER_LIST length, shorter is better */
 	p1cnt = p2cnt = 0;
@@ -256,14 +278,16 @@ prefix_cmp(struct prefix *p1, struct prefix *p2, int *testall)
 		p1cnt = a->len / sizeof(uint32_t);
 	if ((a = attr_optget(asp2, ATTR_CLUSTER_LIST)) != NULL)
 		p2cnt = a->len / sizeof(uint32_t);
-	if ((p2cnt - p1cnt) != 0)
-		return (p2cnt - p1cnt);
+	if (p1cnt < p2cnt)
+		return rv;
+	if (p1cnt > p2cnt)
+		return -rv;
 
 	/* 12. lowest peer address wins (IPv4 is better than IPv6) */
 	if (peer1->remote_addr.aid < peer2->remote_addr.aid)
-		return 1;
+		return rv;
 	if (peer1->remote_addr.aid > peer2->remote_addr.aid)
-		return -1;
+		return -rv;
 	switch (peer1->remote_addr.aid) {
 	case AID_INET:
 		i = memcmp(&peer1->remote_addr.v4, &peer2->remote_addr.v4,
@@ -277,20 +301,41 @@ prefix_cmp(struct prefix *p1, struct prefix *p2, int *testall)
 		fatalx("%s: unknown af", __func__);
 	}
 	if (i < 0)
-		return 1;
+		return rv;
 	if (i > 0)
-		return -1;
+		return -rv;
 
 	/* RFC7911 does not specify this but something like this is needed. */
 	/* 13. lowest path identifier wins */
 	if (p1->path_id < p2->path_id)
-		return 1;
+		return rv;
 	if (p1->path_id > p2->path_id)
-		return -1;
+		return -rv;
 
 	fatalx("Uh, oh a politician in the decision process");
 }
 
+/*
+ * set the dmetric value of np based on the return value of
+ * prefix_evaluate(pp, np) or set it to either PREFIX_DMETRIC_BEST
+ * or PREFIX_DMETRIC_INVALID for the first element.
+ */
+void
+prefix_set_dmetric(struct prefix *pp, struct prefix *np)
+{
+	int testall;
+
+	if (np != NULL) {
+		if (pp == NULL)
+			np->dmetric = prefix_eligible(np) ?
+			    PREFIX_DMETRIC_BEST : PREFIX_DMETRIC_INVALID;
+		else
+			np->dmetric = prefix_cmp(pp, np, &testall);
+		if (np->dmetric < 0)
+			fatalx("bad dmetric in decision process");
+	}
+}
+
 /*
  * Insert a prefix keeping the total order of the list. For routes
  * that may depend on a MED selection the set is scanned until the
@@ -303,7 +348,7 @@ prefix_insert(struct prefix *new, struct prefix *ep, struct rib_entry *re)
 {
 	struct prefix_queue redo = TAILQ_HEAD_INITIALIZER(redo);
 	struct prefix *xp, *np, *insertp = ep;
-	int testall, selected = 0;
+	int testall, preferred, selected = 0, removed = 0;
 
 	/* start scan at the entry point (ep) or the head if ep == NULL */
 	if (ep == NULL)
@@ -312,24 +357,25 @@ prefix_insert(struct prefix *new, struct prefix *ep, struct rib_entry *re)
 	for (xp = ep; xp != NULL; xp = np) {
 		np = TAILQ_NEXT(xp, entry.list.rib);
 
-		if (prefix_cmp(new, xp, &testall) > 0) {
+		if ((preferred = (prefix_cmp(new, xp, &testall) > 0))) {
 			/* new is preferred over xp */
-			if (testall == 0)
-				break;		/* we're done */
-			else if (testall == 2) {
+			if (testall == 2) {
 				/*
 				 * MED inversion, take out prefix and
 				 * put it onto redo queue.
 				 */
 				TAILQ_REMOVE(&re->prefix_h, xp, entry.list.rib);
 				TAILQ_INSERT_TAIL(&redo, xp, entry.list.rib);
-			} else {
+				removed = 1;
+				continue;
+			}
+
+			if (testall == 1) {
 				/*
 				 * lock insertion point and
 				 * continue on with scan
 				 */
 				selected = 1;
-				continue;
 			}
 		} else {
 			/*
@@ -346,12 +392,30 @@ prefix_insert(struct prefix *new, struct prefix *ep, struct rib_entry *re)
 			if (!selected)
 				insertp = xp;
 		}
+
+		/*
+		 * If previous element(s) got removed, fixup the
+		 * dmetric, now that it is clear that this element
+		 * is on the list.
+		 */
+		if (removed) {
+			prefix_set_dmetric(TAILQ_PREV(xp, prefix_queue,
+			    entry.list.rib), xp);
+			removed = 0;
+		}
+
+		if (preferred && testall == 0)
+			break;			/* we're done */
 	}
 
-	if (insertp == NULL)
+	if (insertp == NULL) {
 		TAILQ_INSERT_HEAD(&re->prefix_h, new, entry.list.rib);
-	else
+	} else {
 		TAILQ_INSERT_AFTER(&re->prefix_h, insertp, new, entry.list.rib);
+	}
+
+	prefix_set_dmetric(insertp, new);
+	prefix_set_dmetric(new, TAILQ_NEXT(new, entry.list.rib));
 
 	/* Fixup MED order again. All elements are < new */
 	while (!TAILQ_EMPTY(&redo)) {
@@ -375,11 +439,13 @@ void
 prefix_remove(struct prefix *old, struct rib_entry *re)
 {
 	struct prefix_queue redo = TAILQ_HEAD_INITIALIZER(redo);
-	struct prefix *xp, *np;
-	int testall;
+	struct prefix *xp, *np, *pp;
+	int testall, removed = 0;
 
 	xp = TAILQ_NEXT(old, entry.list.rib);
+	pp = TAILQ_PREV(old, prefix_queue, entry.list.rib);
 	TAILQ_REMOVE(&re->prefix_h, old, entry.list.rib);
+
 	/* check if a MED inversion could be possible */
 	prefix_cmp(old, xp, &testall);
 	if (testall > 0) {
@@ -389,19 +455,36 @@ prefix_remove(struct prefix *old, struct rib_entry *re)
 
 			/* only interested in the testall result */
 			prefix_cmp(old, xp, &testall);
-			if (testall == 0)
-				break;		/* we're done */
-			else if (testall == 2) {
+			if (testall == 2) {
 				/*
 				 * possible MED inversion, take out prefix and
 				 * put it onto redo queue.
 				 */
 				TAILQ_REMOVE(&re->prefix_h, xp, entry.list.rib);
 				TAILQ_INSERT_TAIL(&redo, xp, entry.list.rib);
+				removed = 1;
+				continue;
 			}
+			/*
+			 * If previous element(s) got removed, fixup the
+			 * dmetric, now that it is clear that this element
+			 * is on the list.
+			 */
+			if (removed) {
+				prefix_set_dmetric(TAILQ_PREV(xp, prefix_queue,
+				    entry.list.rib), xp);
+				removed = 0;
+			}
+			if (testall == 0)
+				break;		/* we're done */
 		}
 	}
 
+	if (pp)
+		prefix_set_dmetric(pp, TAILQ_NEXT(pp, entry.list.rib));
+	else
+		prefix_set_dmetric(NULL, TAILQ_FIRST(&re->prefix_h));
+
 	/* Fixup MED order again, reinsert prefixes from the start */
 	while (!TAILQ_EMPTY(&redo)) {
 		xp = TAILQ_FIRST(&redo);
@@ -458,7 +541,7 @@ prefix_best(struct rib_entry *re)
 void
 prefix_evaluate(struct rib_entry *re, struct prefix *new, struct prefix *old)
 {
-	struct prefix	*xp, *active;
+	struct prefix	*newbest, *oldbest;
 	struct rib	*rib;
 
 	rib = re_rib(re);
@@ -466,12 +549,14 @@ prefix_evaluate(struct rib_entry *re, struct prefix *new, struct prefix *old)
 		/* decision process is turned off */
 		if (old != NULL)
 			TAILQ_REMOVE(&re->prefix_h, old, entry.list.rib);
-		if (new != NULL)
+		if (new != NULL) {
 			TAILQ_INSERT_HEAD(&re->prefix_h, new, entry.list.rib);
+			new->dmetric = PREFIX_DMETRIC_INVALID;
+		}
 		return;
 	}
 
-	active = prefix_best(re);
+	oldbest = prefix_best(re);
 
 	if (old != NULL)
 		prefix_remove(old, re);
@@ -479,23 +564,23 @@ prefix_evaluate(struct rib_entry *re, struct prefix *new, struct prefix *old)
 	if (new != NULL)
 		prefix_insert(new, NULL, re);
 
-	xp = TAILQ_FIRST(&re->prefix_h);
-	if (xp != NULL && !prefix_eligible(xp))
-		xp = NULL;
+	newbest = TAILQ_FIRST(&re->prefix_h);
+	if (newbest != NULL && !prefix_eligible(newbest))
+		newbest = NULL;
 
 	/*
 	 * If the active prefix changed or the active prefix was removed
 	 * and added again then generate an update.
 	 */
-	if (active != xp || (old != NULL && xp == old)) {
+	if (oldbest != newbest || (old != NULL && newbest == old)) {
 		/*
-		 * Send update withdrawing re->active and adding xp
-		 * but remember that xp may be NULL aka ineligible.
+		 * Send update withdrawing oldbest and adding newbest
+		 * but remember that newbest may be NULL aka ineligible.
 		 * Additional decision may be made by the called functions.
 		 */
-		rde_generate_updates(rib, xp, active, 0);
+		rde_generate_updates(rib, newbest, oldbest, EVAL_DEFAULT);
 		if ((rib->flags & F_RIB_NOFIB) == 0)
-			rde_send_kroute(rib, xp, active);
+			rde_send_kroute(rib, newbest, oldbest);
 		return;
 	}
 
@@ -506,5 +591,5 @@ prefix_evaluate(struct rib_entry *re, struct prefix *new, struct prefix *old)
 	 */
 	if (rde_evaluate_all())
 		if ((new != NULL && prefix_eligible(new)) || old != NULL)
-			rde_generate_updates(rib, prefix_best(re), NULL, 1);
+			rde_generate_updates(rib, newbest, NULL, EVAL_ALL);
 }
