@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde_rib.c,v 1.238 2022/05/23 13:40:12 deraadt Exp $ */
+/*	$OpenBSD: rde_rib.c,v 1.240 2022/07/08 10:01:52 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Claudio Jeker <claudio@openbsd.org>
@@ -842,7 +842,7 @@ path_put(struct rde_aspath *asp)
 /* prefix specific functions */
 
 static int	prefix_add(struct bgpd_addr *, int, struct rib *,
-		    struct rde_peer *, uint32_t, struct rde_aspath *,
+		    struct rde_peer *, uint32_t, uint32_t, struct rde_aspath *,
 		    struct rde_community *, struct nexthop *,
 		    uint8_t, uint8_t);
 static int	prefix_move(struct prefix *, struct rde_peer *,
@@ -850,7 +850,7 @@ static int	prefix_move(struct prefix *, struct rde_peer *,
 		    struct nexthop *, uint8_t, uint8_t);
 
 static void	prefix_link(struct prefix *, struct rib_entry *,
-		     struct pt_entry *, struct rde_peer *, uint32_t,
+		     struct pt_entry *, struct rde_peer *, uint32_t, uint32_t,
 		     struct rde_aspath *, struct rde_community *,
 		     struct nexthop *, uint8_t, uint8_t);
 static void	prefix_unlink(struct prefix *);
@@ -917,14 +917,14 @@ prefix_get(struct rib *rib, struct rde_peer *peer, uint32_t path_id,
  * Returns NULL if not found.
  */
 struct prefix *
-prefix_adjout_get(struct rde_peer *peer, uint32_t path_id,
+prefix_adjout_get(struct rde_peer *peer, uint32_t path_id_tx,
     struct bgpd_addr *prefix, int prefixlen)
 {
 	struct prefix xp;
 
 	memset(&xp, 0, sizeof(xp));
 	xp.pt = pt_fill(prefix, prefixlen);
-	xp.path_id_tx = path_id;
+	xp.path_id_tx = path_id_tx;
 
 	return RB_FIND(prefix_index, &peer->adj_rib_out, &xp);
 }
@@ -1001,8 +1001,8 @@ prefix_adjout_match(struct rde_peer *peer, struct bgpd_addr *addr)
  */
 int
 prefix_update(struct rib *rib, struct rde_peer *peer, uint32_t path_id,
-    struct filterstate *state, struct bgpd_addr *prefix, int prefixlen,
-    uint8_t vstate)
+    uint32_t path_id_tx, struct filterstate *state, struct bgpd_addr *prefix,
+    int prefixlen, uint8_t vstate)
 {
 	struct rde_aspath	*asp, *nasp = &state->aspath;
 	struct rde_community	*comm, *ncomm = &state->communities;
@@ -1012,6 +1012,8 @@ prefix_update(struct rib *rib, struct rde_peer *peer, uint32_t path_id,
 	 * First try to find a prefix in the specified RIB.
 	 */
 	if ((p = prefix_get(rib, peer, path_id, prefix, prefixlen)) != NULL) {
+		if (path_id_tx != p->path_id_tx)
+			fatalx("path_id mismatch");
 		if (prefix_nexthop(p) == state->nexthop &&
 		    prefix_nhflags(p) == state->nhflags &&
 		    communities_equal(ncomm, prefix_communities(p)) &&
@@ -1044,8 +1046,9 @@ prefix_update(struct rib *rib, struct rde_peer *peer, uint32_t path_id,
 		return (prefix_move(p, peer, asp, comm, state->nexthop,
 		    state->nhflags, vstate));
 	else
-		return (prefix_add(prefix, prefixlen, rib, peer, path_id, asp,
-		    comm, state->nexthop, state->nhflags, vstate));
+		return (prefix_add(prefix, prefixlen, rib, peer, path_id,
+		    path_id_tx, asp, comm, state->nexthop, state->nhflags,
+		    vstate));
 }
 
 /*
@@ -1053,9 +1056,9 @@ prefix_update(struct rib *rib, struct rde_peer *peer, uint32_t path_id,
  */
 static int
 prefix_add(struct bgpd_addr *prefix, int prefixlen, struct rib *rib,
-    struct rde_peer *peer, uint32_t path_id, struct rde_aspath *asp,
-    struct rde_community *comm, struct nexthop *nexthop, uint8_t nhflags,
-    uint8_t vstate)
+    struct rde_peer *peer, uint32_t path_id, uint32_t path_id_tx,
+    struct rde_aspath *asp, struct rde_community *comm,
+    struct nexthop *nexthop, uint8_t nhflags, uint8_t vstate)
 {
 	struct prefix		*p;
 	struct rib_entry	*re;
@@ -1065,8 +1068,8 @@ prefix_add(struct bgpd_addr *prefix, int prefixlen, struct rib *rib,
 		re = rib_add(rib, prefix, prefixlen);
 
 	p = prefix_alloc();
-	prefix_link(p, re, re->prefix, peer, path_id, asp, comm, nexthop,
-	    nhflags, vstate);
+	prefix_link(p, re, re->prefix, peer, path_id, path_id_tx, asp, comm,
+	    nexthop, nhflags, vstate);
 
 	/* add possible pftable reference form aspath */
 	if (asp && asp->pftableid)
@@ -1094,8 +1097,8 @@ prefix_move(struct prefix *p, struct rde_peer *peer,
 
 	/* add new prefix node */
 	np = prefix_alloc();
-	prefix_link(np, prefix_re(p), p->pt, peer, p->path_id, asp, comm,
-	    nexthop, nhflags, vstate);
+	prefix_link(np, prefix_re(p), p->pt, peer, p->path_id, p->path_id_tx,
+	    asp, comm, nexthop, nhflags, vstate);
 
 	/* add possible pftable reference from new aspath */
 	if (asp && asp->pftableid)
@@ -1165,14 +1168,14 @@ prefix_add_eor(struct rde_peer *peer, uint8_t aid)
  * Put a prefix from the Adj-RIB-Out onto the update queue.
  */
 void
-prefix_adjout_update(struct rde_peer *peer, struct filterstate *state,
-    struct bgpd_addr *prefix, int prefixlen, uint8_t vstate)
+prefix_adjout_update(struct prefix *p, struct rde_peer *peer,
+    struct filterstate *state, struct bgpd_addr *prefix, int prefixlen,
+    uint32_t path_id_tx, uint8_t vstate)
 {
 	struct rde_aspath *asp;
 	struct rde_community *comm;
-	struct prefix *p;
 
-	if ((p = prefix_adjout_get(peer, 0, prefix, prefixlen)) == NULL) {
+	if (p == NULL) {
 		p = prefix_alloc();
 		/* initally mark DEAD so code below is skipped */
 		p->flags |= PREFIX_FLAG_ADJOUT | PREFIX_FLAG_DEAD;
@@ -1182,7 +1185,7 @@ prefix_adjout_update(struct rde_peer *peer, struct filterstate *state,
 			p->pt = pt_add(prefix, prefixlen);
 		pt_ref(p->pt);
 		p->peer = peer;
-		p->path_id_tx = 0 /* XXX force this for now */;
+		p->path_id_tx = path_id_tx;
 
 		if (RB_INSERT(prefix_index, &peer->adj_rib_out, p) != NULL)
 			fatalx("%s: RB index invariant violated", __func__);
@@ -1191,7 +1194,14 @@ prefix_adjout_update(struct rde_peer *peer, struct filterstate *state,
 	if ((p->flags & PREFIX_FLAG_ADJOUT) == 0)
 		fatalx("%s: prefix without PREFIX_FLAG_ADJOUT hit", __func__);
 	if ((p->flags & (PREFIX_FLAG_WITHDRAW | PREFIX_FLAG_DEAD)) == 0) {
-		if (prefix_nhflags(p) == state->nhflags &&
+		/*
+		 * XXX for now treat a different path_id_tx like different
+		 * attributes and force out an update. It is unclear how
+		 * common it is to have equivalent updates from alternative
+		 * paths.
+		 */
+		if (p->path_id_tx == path_id_tx &&
+		    prefix_nhflags(p) == state->nhflags &&
 		    prefix_nexthop(p) == state->nexthop &&
 		    communities_equal(&state->communities,
 		    prefix_communities(p)) &&
@@ -1221,6 +1231,15 @@ prefix_adjout_update(struct rde_peer *peer, struct filterstate *state,
 	/* nothing needs to be done for PREFIX_FLAG_DEAD and STALE */
 	p->flags &= ~PREFIX_FLAG_MASK;
 
+	/* update path_id_tx now that the prefix is unlinked */
+	if (p->path_id_tx != path_id_tx) {
+		/* path_id_tx is part of the index so remove and re-insert p */
+		RB_REMOVE(prefix_index, &peer->adj_rib_out, p);
+		p->path_id_tx = path_id_tx;
+		if (RB_INSERT(prefix_index, &peer->adj_rib_out, p) != NULL)
+			fatalx("%s: RB index invariant violated", __func__);
+	}
+
 	if ((asp = path_lookup(&state->aspath)) == NULL) {
 		/* Path not available, create and link a new one. */
 		asp = path_copy(path_get(), &state->aspath);
@@ -1232,8 +1251,8 @@ prefix_adjout_update(struct rde_peer *peer, struct filterstate *state,
 		comm = communities_link(&state->communities);
 	}
 
-	prefix_link(p, NULL, p->pt, peer, 0, asp, comm, state->nexthop,
-	    state->nhflags, vstate);
+	prefix_link(p, NULL, p->pt, peer, 0, p->path_id_tx, asp, comm,
+	    state->nexthop, state->nhflags, vstate);
 	peer->prefix_out_cnt++;
 
 	if (p->flags & PREFIX_FLAG_MASK)
@@ -1549,9 +1568,9 @@ prefix_destroy(struct prefix *p)
  */
 static void
 prefix_link(struct prefix *p, struct rib_entry *re, struct pt_entry *pt,
-    struct rde_peer *peer, uint32_t path_id, struct rde_aspath *asp,
-    struct rde_community *comm, struct nexthop *nexthop, uint8_t nhflags,
-    uint8_t vstate)
+    struct rde_peer *peer, uint32_t path_id, uint32_t path_id_tx,
+    struct rde_aspath *asp, struct rde_community *comm,
+    struct nexthop *nexthop, uint8_t nhflags, uint8_t vstate)
 {
 	if (re)
 		p->entry.list.re = re;
@@ -1560,6 +1579,7 @@ prefix_link(struct prefix *p, struct rib_entry *re, struct pt_entry *pt,
 	p->peer = peer;
 	p->pt = pt_ref(pt);
 	p->path_id = path_id;
+	p->path_id_tx = path_id_tx;
 	p->validation_state = vstate;
 	p->nhflags = nhflags;
 	p->nexthop = nexthop_ref(nexthop);
